#  ooo  oooo  oooo o   o     o    ooo   ooo  o   o  oooo ooooo o  ooo   oooo
# 0   0 0   0 0    00  0    0 0  0   0 0   0 0   0 0       0   0 0   0 0
# 0   0 oooo  oooo 0 0 0   0   0 0     0   0 0   0  oooo   0   0 0      oooo
# 0   0 o     0    0 o 0   ooooo 0   o 0   0 0   0      0  0   0 0   o      0
#  ooo  o     oooo o  0o  0     0 ooo   ooo   ooo  0oooo   0   0  ooo  0oooo
#
#  (  (  (  (  (  (  (  (  (  (  (  (  O  )  )  )  )  )  )  )  )  )  )  )  )
#
#    OpenAcoustics: The Open Source Collection for Computational Acoustics
#
#               Copyright (C) 2008-2009 The OpenAcoustics Team
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

## @package openacoustics.bem
# 2 dimensional bem module for acoustic problem

__author__ = "hisaha <hisaha@souldsp.jp>"
__date__ = "2008/02/07"
__credits__ = "Hisharu Suzuki"
__version__ = "$Revision: 0 $"


from scipy import *
from scipy.special import *
from scipy.integrate import *
from pylab import *
from openacoustics.gmsh import *

## for calculating 2-dimensional sound field with bem
class Bem2D:
    """A simple 2-dimensional bem class for calculating sound field with line element"""
    
    ## partial difference of green function in normal direction
    #  
    #  \f[
    #  \exp{2\pi}
    #  \f]
    #  @param k wave number
    #  @param rm cordinate vector of middle point of m-th elemtn
    #  @param rn cordinate vector of middle point of n-th elemtn
    #  @param en normal vector of n-th element
    #  @retval val calculated value
    def Gn(self,k,rm,rn,en):
        mn = rn - rm
        r = linalg.norm(mn)

        H = jv(1, k * r) - 1j * yv(1, k * r)
        cos_theta = dot(mn, en) / linalg.norm(mn) / linalg.norm(en)
        val = H * cos_theta * k / (-4j)

        #print theta

        return val

    ## green function
    #  @param k wave number
    #  @param rm cordinate vector of middle point of m-th element
    #  @param rn cordinate vector of middle point of n-th element
    #  @retval val calculated value
    def G(self,k, rm, rn):
        r = linalg.norm(rn - rm)
        H = jv(0, k * r) - 1j * yv(0, k * r)
        val = H / (-4j)
        return val

    ## create A matrix
    #  @param r cordinates of middle point of elements
    #  @param nv normal vector of elements
    #  @param S areas of surfaces
    #  @param k wave number
    #  @param w angular frequency
    #  @param ro0 density of air
    #  @param beta admittance of whole element
    #  @retval A calculated A matrix
    def createMatrix(self,r,nv,S,k,w,ro0,beta):
        M=nv.shape[0]
        A = zeros((M, M), 'complex64')
        
        #Create matrix A
        for m in range(M):
            for n in range(M):
                en = nv[n, :]
                if m == n:
                    A[m, n] = -1j * w * ro0 * beta
                else:
                    #val = Gn(k, r[m,:], r[n,:], en)
                    A[m, n] = (-1j * w * ro0 * beta * self.G(k, r[m, :], r[n, :])
                                - self.Gn(k, r[m, :], r[n, :], en)) * S[n]
        ##            node1 = nodes[sf[ii, 0], :]
        ##            node2 = nodes[sf[ii, 1], :]
        ##            f = lambda y, x:G(k, r[m, :], array([x, y]))
        ##            g = lambda y, x:Gn(k, r[m, :], array([x, y]), en)
        ##            Gi, err = dblquad(f, node1[1], node2[1], lambda x:node1[0],
        ##                              lambda x:node2[0])
        ##            Gni, err = dblquad(g, node1[1], node2[1], lambda x:node1[0],
        ##                               lambda x:node2[0])
        ##            A[m, n] = -1j * w * ro0 * beta * Gi
        ##            A[m, n] -= Gni
        
        return A
    
    ## create matrix of sound sources
    # @param rs cordinates of points of sources
    # @param r cordinates of middle points of elements
    # @param k wave number
    def createSourceMatrix(self,rs,r,k):
        """TODO: multiple source is not implemented"""
        M=r.shape[0]
        Pd = zeros(M, "complex64")
        # Creation of direct wave matrix
        for m in range(M):
            Pd[m] = self.G(k, rs, r[m, :])
            
        return Pd

    def calcGrid(self,Phi,xx,yy,rs,r,nv,S,k,w,ro0,beta):
        M = r.shape[0]
        An = zeros(M,'complex64')
        Pr = zeros((len(xx), len(yy)), 'complex64')

        for ii in range(len(xx)):
            for jj in range(len(yy)):
                rr = array([xx[ii], yy[jj]])
                pd = self.G(k, rs, rr)

                for n in range(M):
                    en = nv[n, :]
                    An[n] = (-1j * w * beta * ro0 * self.G(k, rr, r[n, :])
                              - self.Gn(k, rr, r[n, :], en)) * S[n]

                Pr[ii, jj] = pd + dot(An, Phi)
                
        return Pr

#
#class Bem3D:
#    def Gn(k,rm,rn,en):
#        mn = rn - rm
#        r = linalg.norm(mn)
#
#        H = jv(1, k * r) - 1j * yv(1, k * r)
#        cos_theta = dot(mn, en) / linalg.norm(mn) / linalg.norm(en)
#        val = H * cos_theta * k / (-4j)
#
#        #print theta
#
#        return val
#
#    def G(k, rm, rn):
#        r = linalg.norm(rn - rm)
#        H = jv(0, k * r) - 1j * yv(0, k * r)
#        val = H / (-4j)
#        return val
