#  ooo  oooo  oooo o   o     o    ooo   ooo  o   o  oooo ooooo o  ooo   oooo
# 0   0 0   0 0    00  0    0 0  0   0 0   0 0   0 0       0   0 0   0 0
# 0   0 oooo  oooo 0 0 0   0   0 0     0   0 0   0  oooo   0   0 0      oooo
# 0   0 o     0    0 o 0   ooooo 0   o 0   0 0   0      0  0   0 0   o      0
#  ooo  o     oooo o  0o  0     0 ooo   ooo   ooo  0oooo   0   0  ooo  0oooo
#
#  (  (  (  (  (  (  (  (  (  (  (  (  O  )  )  )  )  )  )  )  )  )  )  )  )
#
#    OpenAcoustics: The Open Source Collection for Computational Acoustics
#
#               Copyright (C) 2008-2009 The OpenAcoustics Team
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


__author__ = "t_ishiduka <http://sourceforge.net.users.t_ishiduka>" 
__date__ = "2010/08/19"
__credits__ = "ISHIZUKA Takashi"
__version__ = "$Revision: 0 $"

# import modules 
from scipy import array, linalg, zeros, dot, pi, sqrt, special
from scipy.integrate import quadrature


#=========================================================================
#                        Definitions of functions
#=========================================================================
# Hankel function of the first kind of order zero
def h01(x):
    return special.j0(x) + 1j * special.y0(x)


# Hankel function of the first kind of order one 
def h11(x):
    return special.j1(x) + 1j * special.y1(x)


# Green function of the 2-dimentional free field
# See eqn. (2.33)
def GreenFunction(receiver, source, waveNumber):
    r = receiver - source
    abs_r = sqrt(r[0] ** 2. + r[1] ** 2.)

    return 1j * h01(abs_r * waveNumber) / 4.


# Integrand of element integral Ia,
# on local axis xi.
# See eqn. (2.54)
def integrandIa(xi, element, receiver, waveNumber):
    r_x = receiver[0] - element['middlePoint'][0] + element['localAxis'][0] * xi
    r_y = receiver[1] - element['middlePoint'][1] + element['localAxis'][1] * xi
    abs_r = sqrt(r_x ** 2. + r_y ** 2.)
    dr_dn = - (r_x * element['normalVector'][0] + r_y * element['normalVector'][1]) \
            / abs_r

    return h11(abs_r * waveNumber) * dr_dn


# Integrand of element integral Ib,
# on local axis xi.
# See eqn. (2.55)
def integrandIb( xi, element, receiver, waveNumber ):
    r_x = receiver[0] - element['middlePoint'][0] + element['localAxis'][0] * xi
    r_y = receiver[1] - element['middlePoint'][1] + element['localAxis'][1] * xi
    abs_r = sqrt(r_x ** 2. + r_y ** 2.)

    return h01(abs_r * waveNumber)


# Singular integral of the Hankel function of the first kind of order zero
# Int_{0}^{a} h01(t) dt
# See eqn (2.39)
def singularIntegralH01(a):
    h01a = h01(a);
    h11a = h11(a);
    H0a = special.struve(0,a);
    H1a = special.struve(1,a);

    value = a * h01a + pi * a / 2. * (H0a * h11a - H1a * h01a);
    
    return value


#=========================================================================
#                              Main routine
#=========================================================================
#========================================================== Initialization 
#-- sound speed -- assuming temperature = 15 in Celsius degree.
soundSpeed = 340.

#-- frequency --
frequency = 100.
angularFrequency = 2. * pi * frequency
waveNumber = angularFrequency / soundSpeed

#-- boundary elements --
nElement = 4
element = [{ 'p1': array([.5 , .5 ]), 'p2': array([-.5, .5 ]), 'admittance': 0. },
           { 'p1': array([-.5, .5 ]), 'p2': array([-.5, -.5]), 'admittance': 0. },
           { 'p1': array([-.5, -.5]), 'p2': array([.5,  -.5]), 'admittance': 1. },
           { 'p1': array([.5 , -.5]), 'p2': array([.5,  .5 ]), 'admittance': 0. }
          ]

#-- source & receivers --
source = array( [0.4, 0.4] )

receiver = array( [ [0.2 ,  0.2],
                  [-0.2,  0.2],
                  [-0.2, -0.2],
                  [0.2 , -0.2] ] )

#======================================= Preparation of element parameters 
print 'boundary Elements:'
for n in range(nElement):
    element[n]['middlePoint'] = (element[n]['p1'] + element[n]['p2']) / 2.
    element[n]['localAxis'] = (element[n]['p2'] - element[n]['p1']) / 2.
    element[n]['length'] = sqrt(element[n]['localAxis'][0] ** 2. \
            + element[n]['localAxis'][1] ** 2.) * 2.
    element[n]['normalVector'] = \
            array([ element[n]['localAxis'][1], -element[n]['localAxis'][0] ]) \
            * 2. / element[n]['length']

    # show element parameters
    print 'element[', n, ']:', 'p1', element[n]['p1'],\
            ', p2', element[n]['p2'], ', admittance', element[n]['admittance'], ','
    print 'middlePoint', element[n]['middlePoint'], \
            ', localAxis:', element[n]['localAxis'], \
            ', length:', element[n]['length'],\
            ', normalVector:', element[n]['normalVector']
print

#============================================ Build coefficient matrix 'A'
#See eqn. (2.24) and (2.22)
A = zeros((nElement,nElement), 'complex64')

# loop for boundary elements
j_4 = 1j / 4.
for n in range(nElement): 
    a = -1j * waveNumber * element[n]['admittance']
    halfLength = element[n]['length'] / 2.

    # loop for receivers at middle points of each element
    for m in range(nElement): 
        rm = element[m]['middlePoint']

        if m != n: # general components
            #See eqn. (2.54)
            Ia = - j_4 * waveNumber * halfLength \
                    * quadrature(integrandIa, -1, 1, args=(element[n],rm,waveNumber))[0]
            #See eqn. (2.55)
            Ib = j_4 * halfLength  \
                    * quadrature(integrandIb, -1, 1, args=(element[n],rm,waveNumber))[0]
            A[m,n] = Ia + a * Ib

        else: # diagonal components : take singular integral
            #See eqn. (2.46)
            Ib = (1j / (2. * waveNumber)) * \
                    singularIntegralH01(waveNumber * halfLength)
            A[m,n] = 1./2. + a * Ib

# show A
print 'Coefficient matrix A:'
print A
print

#=========================================== Build right hand vector 'psy'
#See eqn. (2.23) and (2.33)
psi = zeros((nElement,1), 'complex64')

for n in range(nElement):
    psi[n] = GreenFunction( element[n]['middlePoint'], source, waveNumber )

#show psy
print 'Right hand side vector psi:'
print psi
print

#===================================== Solve linear equation A * phi = psy
#See eqn. (2.24)
phi = linalg.solve( A, psi )

#show phi
print 'Solution of linear equation phi (A*phi=psy):'
print phi
print

#============================== Calculate velocity potentials at receivers
#See eqn. (2.15) and (2.50)
phiR = zeros((receiver.shape[0],1), 'complex64' )
An = zeros(nElement, 'complex64')

for k in range(receiver.shape[0]): # loop for receivers
    Rk = receiver[k,:]
    
    for n in range(nElement):
        a = -1j * waveNumber * element[n]['admittance']
        halfLength = element[n]['length'] / 2.

        #See eqn. (2.54)
        Ia = -j_4 * waveNumber * halfLength \
                * quadrature(integrandIa, -1, 1, args=(element[n],Rk,waveNumber))[0]
        #See eqn. (2.55)
        Ib = j_4 * halfLength \
                * quadrature(integrandIb, -1, 1, args=(element[n],Rk,waveNumber))[0]
        An[n] = Ia + a * Ib

    #See eqn. (2.50) 
    phiR[k] = GreenFunction(Rk, source, waveNumber) - dot(An, phi)[0]

#show results
print 'Velocity potentials at receivers:'
print phiR
print

raw_input("Press ENTER to exit")

# End






