#  ooo  oooo  oooo o   o     o    ooo   ooo  o   o  oooo ooooo o  ooo   oooo
# 0   0 0   0 0    00  0    0 0  0   0 0   0 0   0 0       0   0 0   0 0
# 0   0 oooo  oooo 0 0 0   0   0 0     0   0 0   0  oooo   0   0 0      oooo
# 0   0 o     0    0 o 0   ooooo 0   o 0   0 0   0      0  0   0 0   o      0
#  ooo  o     oooo o  0o  0     0 ooo   ooo   ooo  0oooo   0   0  ooo  0oooo
#
#  (  (  (  (  (  (  (  (  (  (  (  (  O  )  )  )  )  )  )  )  )  )  )  )  )
#
#    OpenAcoustics: The Open Source Collection for Computational Acoustics
#
#               Copyright (C) 2008-2009 The OpenAcoustics Team
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

## @package openacoustics.fdtd
# 1-3 dimensional fdtd module for acoustic problem

__author__ = "hisaha <hisaha@souldsp.jp>"
__date__ = "2008/12/23"
__credits__ = "Hisharu Suzuki"
__version__ = "$Revision: 0 $"

from pylab import *
from scipy import *
import scipy.weave
import time

## for calculating 1-3 dimensional sound field with fdtd
class Fdtd:
    """A simple 1, 2, 3 dimensional fdtd class"""
    
    ## for allocation of required arrays for fdtd method
    # @param XYZ tuple of number of elements in each direction
    # @retval K bulk modulus filled by zero
    # @retval Ro density filled by zero
    # @retval zp potential resistance filled by zero
    # @retval zv flow resistance filled by zero
    # @retval P1 arrays of sound pressure ( current in time )
    # @retval P2 arrays of sound pressure ( next in time )
    # @retval V1 arrays of particle velocity ( current in time )
    # @retval V2 arrays of particle velocity ( next in time )
    def allocateArrays(self,XYZ):
        P1=[]
        P2=[]
        V1=[]
        V2=[]
        for i in range(len(XYZ)):
            P1.append( zeros(XYZ, "float64" ) )
            P2.append( zeros(XYZ, "float64" ) )
            V1.append( zeros(XYZ, "float64" ) )
            V2.append( zeros(XYZ, "float64" ) )
        
        K = zeros(XYZ, "float64")
        Ro = zeros(XYZ, "float64")
        zp = zeros(XYZ, "float64")
        zv = zeros(XYZ, "float64")

        return K,Ro,zp,zv,P1,P2,V1,V2
    
    ## for update sound field
    # @param K bulk modulus
    # @param Ro density
    # @param zp potential resistance
    # @param zv flow resistance
    # @param P1 arrays of sound pressure ( current in time )
    # @param P2 arrays of sound pressure ( next in time )
    # @param V1 arrays of particle velocity ( current in time )
    # @param V2 arrays of particle velocity ( next in time )
    # @retval P1 arrays of updated sound pressure ( current in time )
    # @retval P2 arrays of updated sound pressure ( next in time )
    # @retval V1 arrays of updated particle velocity ( current in time )
    # @retval V2 arrays of updated particle velocity ( next in time )
    def update(self, dt, dxyz, K, Ro, zp, zv, P1, P2, V1, V2):
        return self.updates(1, dt, dxyz, K, Ro, zp, zv, P1, P2, V1, V2)
    
    def lightupdate(self, dt, dxyz, K, Ro, zp, zv, P1, P2, V1, V2):
        return self.lightupdates(1, dt, dxyz, K, Ro, zp, zv, P1, P2, V1, V2)
        
    def updates(self, loop, dt, dxyz, K, Ro, zp, zv, P1, P2, V1, V2):
        if len(dxyz) == 1:
            self.update1(loop, dt, dxyz, K, Ro, zp, zv, P1, P2, V1, V2)

        if len(dxyz) == 2:
            P1[0], P2[0], P1[1], P2[1], V1[0], V2[0], V1[1], V2[1] \
                = self.update2(loop, float(dt), dxyz, K, Ro, zp, zv, P1[0],
                               P2[0], P1[1], P2[1], V1[0], V2[0], V1[1], V2[1])
            return P1, P2, V1, V2

        if len(dxyz) == 3:
            self.update3(loop, dt, dxyz, K, Ro, zp, zv, P1, P2, V1, V2)
            
    def lightupdates(self, loop, dt, dxyz, K, Ro, zp, zv, P1, P2, V1, V2):
        if len(dxyz) == 2:
            P1, P2, V1[0], V2[0], V1[1], V2[1] \
                = self.lightupdate2(loop, float(dt), dxyz, K, Ro, zp, zv, P1,
                                    P2, V1[0], V2[0], V1[1], V2[1])
            return P1, P2, V1, V2

    def update2(self, loop, dt, dxyz, K, Ro, zp, zv, Px1, Px2, Py1, Py2, Vx1,
                Vx2, Vy1, Vy2):
        code = " \
        \n\
        blitz::Range I(0, Px1.shape()[0] - 2), J(0, Px1.shape()[1] - 2); \n\
        for (int l = 0; l < loop; l++)\n\
        {\n\
            Vx2(I + 1, J) = Vx1(I+1, J) - dt / (0.5 * (Ro(I + 1, J) + Ro(I, J))) / dxyz(0) * (Px1(I + 1, J) - Px1(I, J) + Py1(I + 1, J) - Py1(I, J)); \n\
            Vy2(I, J + 1) = Vy1(I, J + 1) - dt / (0.5 * (Ro(I, J + 1) + Ro(I, J))) / dxyz(1) * (Px1(I, J + 1) - Px1(I, J) + Py1(I, J + 1) - Py1(I, J)); \n\
            Px2(I, J) = Px1(I, J) - dt * K(I, J) /dxyz(0) * (Vx2(I + 1,J) - Vx2(I,J)); \n\
            Py2(I, J) = Py1(I, J) - dt * K(I, J) /dxyz(1) * (Vy2(I, J + 1) - Vy2(I,J)); \n\
            \n\
            blitz::cycleArrays(Px1, Px2); \n\
            blitz::cycleArrays(Py1, Py2); \n\
            blitz::cycleArrays(Vx1, Vx2); \n\
            blitz::cycleArrays(Vy1, Vy2); \n\
        }\n\
        "
        scipy.weave.inline(code, ['loop', 'dt', 'dxyz', 'K', 'Ro', 'zp', 'zv',
                                  'Px1', 'Px2', 'Py1', 'Py2', 'Vx1', 'Vx2',
                                  'Vy1', 'Vy2'],
                           type_converters = scipy.weave.converters.blitz)
        return Px2, Px1, Py2, Py1, Vx2, Vx1, Vy2, Vy1
    
    def lightupdate2(self, loop, dt, dxyz, K, Ro, zp, zv, P1, P2, Vx1, Vx2,
                     Vy1, Vy2):
        code = " \
        \
        blitz::Range I(0, P1.shape()[0] - 2), J(0, P1.shape()[1] - 2); \
        for (int l = 0; l < loop; l++)\
        {\
            Vx2(I + 1,J) = Vx1(I + 1,J) - dt / Ro(I, J) / dxyz(0) * (P1(I + 1, J) - P1(I, J)); \
            Vy2(I, J + 1) = Vy1(I, J + 1) - dt / Ro(I, J + 1) / dxyz(1) * (P1(I,J + 1) - P1(I,J)); \
            P2(I, J) = P1(I, J) - dt * K(I, J) / dxyz(0) * (Vx2(I + 1, J) - Vx2(I, J) + Vy2(I, J + 1) - Vy2(I, J)); \
            \
            blitz::cycleArrays(P1, P2); \
            blitz::cycleArrays(Vx1, Vx2); \
            blitz::cycleArrays(Vy1, Vy2); \
        }\
        "
        scipy.weave.inline(code, ['loop', 'dt', 'dxyz', 'K', 'Ro', 'zp', 'zv',
                                  'P1', 'P2', 'Vx1', 'Vx2', 'Vy1', 'Vy2'],
                           type_converters = scipy.weave.converters.blitz)
        return P2, P1, Vx2, Vx1, Vy2, Vy1

