#  ooo  oooo  oooo o   o     o    ooo   ooo  o   o  oooo ooooo o  ooo   oooo
# 0   0 0   0 0    00  0    0 0  0   0 0   0 0   0 0       0   0 0   0 0
# 0   0 oooo  oooo 0 0 0   0   0 0     0   0 0   0  oooo   0   0 0      oooo
# 0   0 o     0    0 o 0   ooooo 0   o 0   0 0   0      0  0   0 0   o      0
#  ooo  o     oooo o  0o  0     0 ooo   ooo   ooo  0oooo   0   0  ooo  0oooo
#
#  (  (  (  (  (  (  (  (  (  (  (  (  O  )  )  )  )  )  )  )  )  )  )  )  )
#
#    OpenAcoustics: The Open Source Collection for Computational Acoustics
#
#               Copyright (C) 2008-2009 The OpenAcoustics Team
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

import math as ma
import numpy as np


## Air dencity
#  under a condition of specific temperature.
#  Ref.:田中俊六 他, 最新 建築環境工学 [改訂2版],
#       井上書院, ISBN 4-7530-1734-6, p. 149.
#  @param temperature (in Celsius degree), default=15.0
#  @param static air pressure (N/m2), default=101325.0
#  @retval density of the air (kg/m3)
def airDensity(temp=15., p=101325.):
    """Air density"""

    rho = 353.25 / (273.15 + temp) * (p / 101325.)
    return rho


## Sound speed in air
#  Sound speed in air under a condition of specific temperature.
#  Ref.: P. M. Morse, Vibration and sound (2nd Edition), 
#        Acoust. Soc. Am., ISBN 0-88318-876-7, p. 222.
#  @param temperature (in Celsius degree), default=15.0
#  @param static air pressure (N/m2), default=101325.0
#  @retval sound speed (m/s) 
def soundSpeed(temp=15., p=101325.):
    """Sound speed in air"""
    
    c = ma.sqrt( 1.40 * p / airDensity(temp,p) )

    return c

## Constants for a porous medium
#  Estimate propagation constant and characteristic impedance
#  for a porous medium (e.g. a glass wool) by Miki's model.
#  
#  Ref.: Y. Miki, Acoustical properties of porous materials 
#        - Modifications of Delany-Bazles models -.
#        J. Acost. Soc. Am., 74(5), pp. 1577-1582, 1983.
#
#  NOTE: assuming terms on time to exp(-j \omega t)
#
#  @param frequency
#  @param flow resistaivity
#  @param temperature (in Celsius degree), default=15.0
#  @retval propation constant
#  @retval characteristic impedance
def mikiModel(f, sigma, temp=15.):
    """Constants for a porous medium"""

    const1 = (f / sigma) ** (-0.618)
    const2 = (f / sigma) ** (-0.632)
    c = soundVelosity(temp)
    rho = airDensity(temp)

    g = 1j*(2.*pi*f/c) * (( 1. + 0.109 * const1) - 1j * 0.160 * const1)
    z = rho * c * ((1. + 0.07 * const2) - 1j * 0.107 * const2)
    # adjust to terms on time exp(-j\omega t)
    g = np.conj(g)
    z = np.conj(z)

    return g, z
    

